@@ -18,6 +18,8 @@ if TYPE_CHECKING:
     import sys
     from types import TracebackType
 
+    from ._read_write import ReadWriteLock
+
     if sys.version_info >= (3, 11):  # pragma: no cover (py311+)
         from typing import Self
     else:  # pragma: no cover (<py311)
@@ -33,10 +35,10 @@ _LOGGER = logging.getLogger("filelock")
 class AcquireReturnProxy:
     """A context-aware object that will release the lock file when exiting."""
 
-    def __init__(self, lock: BaseFileLock) -> None:
-        self.lock = lock
+    def __init__(self, lock: BaseFileLock | ReadWriteLock) -> None:
+        self.lock: BaseFileLock | ReadWriteLock = lock
 
-    def __enter__(self) -> BaseFileLock:
+    def __enter__(self) -> BaseFileLock | ReadWriteLock:
         return self.lock
 
     def __exit__(
@@ -79,6 +81,8 @@ class ThreadLocalFileContext(FileLockContext, local):
 
 
 class FileLockMeta(ABCMeta):
+    _instances: WeakValueDictionary[str, BaseFileLock]
+
     def __call__(  # noqa: PLR0913
         cls,
         lock_file: str | os.PathLike[str],
@@ -91,7 +95,7 @@ class FileLockMeta(ABCMeta):
         **kwargs: Any,  # capture remaining kwargs for subclasses  # noqa: ANN401
     ) -> BaseFileLock:
         if is_singleton:
-            instance = cls._instances.get(str(lock_file))  # type: ignore[attr-defined]
+            instance = cls._instances.get(str(lock_file))
             if instance:
                 params_to_check = {
                     "thread_local": (thread_local, instance.is_thread_local()),
@@ -106,7 +110,7 @@ class FileLockMeta(ABCMeta):
                     if passed_param != set_param
                 }
                 if not non_matching_params:
-                    return cast(BaseFileLock, instance)
+                    return cast("BaseFileLock", instance)
 
                 # parameters do not match; raise error
                 msg = "Singleton lock instances cannot be initialized with differing arguments"
@@ -120,7 +124,6 @@ class FileLockMeta(ABCMeta):
         # (https://github.com/tox-dev/filelock/pull/340)
 
         all_params = {
-            "lock_file": lock_file,
             "timeout": timeout,
             "mode": mode,
             "thread_local": thread_local,
@@ -129,21 +132,25 @@ class FileLockMeta(ABCMeta):
             **kwargs,
         }
 
-        present_params = set(inspect.signature(cls.__init__).parameters)  # type: ignore[misc]
+        present_params = inspect.signature(cls.__init__).parameters
         init_params = {key: value for key, value in all_params.items() if key in present_params}
-        # The `lock_file` parameter is required
-        init_params["lock_file"] = lock_file
 
-        instance = super().__call__(**init_params)
+        instance = super().__call__(lock_file, **init_params)
 
         if is_singleton:
-            cls._instances[str(lock_file)] = instance  # type: ignore[attr-defined]
+            cls._instances[str(lock_file)] = instance
 
-        return cast(BaseFileLock, instance)
+        return cast("BaseFileLock", instance)
 
 
 class BaseFileLock(contextlib.ContextDecorator, metaclass=FileLockMeta):
-    """Abstract base class for a file lock object."""
+    """
+    Abstract base class for a file lock object.
+
+    Provides a reentrant, cross-process exclusive lock backed by OS-level primitives. Subclasses implement the
+    actual locking mechanism (:class:`UnixFileLock <filelock.UnixFileLock>`,
+    :class:`WindowsFileLock <filelock.WindowsFileLock>`, :class:`SoftFileLock <filelock.SoftFileLock>`).
+    """
 
     _instances: WeakValueDictionary[str, BaseFileLock]
 
@@ -166,16 +173,16 @@ class BaseFileLock(contextlib.ContextDecorator, metaclass=FileLockMeta):
         Create a new lock object.
 
         :param lock_file: path to the file
-        :param timeout: default timeout when acquiring the lock, in seconds. It will be used as fallback value in \
-            the acquire method, if no timeout value (``None``) is given. If you want to disable the timeout, set it \
+        :param timeout: default timeout when acquiring the lock, in seconds. It will be used as fallback value in
+            the acquire method, if no timeout value (``None``) is given. If you want to disable the timeout, set it
             to a negative value. A timeout of 0 means that there is exactly one attempt to acquire the file lock.
         :param mode: file permissions for the lockfile
-        :param thread_local: Whether this object's internal context should be thread local or not. If this is set to \
-            ``False`` then the lock will be reentrant across threads.
+        :param thread_local: Whether this object's internal context should be thread local or not. If this is set
+            to ``False`` then the lock will be reentrant across threads.
         :param blocking: whether the lock should be blocking or not
-        :param is_singleton: If this is set to ``True`` then only one instance of this class will be created \
-            per lock file. This is useful if you want to use the lock object for reentrant locking without needing \
-            to pass the same object around.
+        :param is_singleton: If this is set to ``True`` then only one instance of this class will be created per
+            lock file. This is useful if you want to use the lock object for reentrant locking without needing to
+            pass the same object around.
 
         """
         self._is_thread_local = thread_local
@@ -197,7 +204,11 @@ class BaseFileLock(contextlib.ContextDecorator, metaclass=FileLockMeta):
 
     @property
     def is_singleton(self) -> bool:
-        """:return: a flag indicating if this lock is singleton or not"""
+        """
+        :return: a flag indicating if this lock is singleton or not
+
+        .. versionadded:: 3.13.0
+        """
         return self._is_singleton
 
     @property
@@ -226,7 +237,11 @@ class BaseFileLock(contextlib.ContextDecorator, metaclass=FileLockMeta):
 
     @property
     def blocking(self) -> bool:
-        """:return: whether the locking is blocking or not"""
+        """
+        :return: whether the locking is blocking or not
+
+        .. versionadded:: 3.14.0
+        """
         return self._context.blocking
 
     @blocking.setter
@@ -257,7 +272,6 @@ class BaseFileLock(contextlib.ContextDecorator, metaclass=FileLockMeta):
     @property
     def is_locked(self) -> bool:
         """
-
         :return: A boolean indicating if the lock file is holding the lock currently.
 
         .. versionchanged:: 2.0.0
@@ -282,12 +296,12 @@ class BaseFileLock(contextlib.ContextDecorator, metaclass=FileLockMeta):
         """
         Try to acquire the file lock.
 
-        :param timeout: maximum wait time for acquiring the lock, ``None`` means use the default :attr:`~timeout` is and
-         if ``timeout < 0``, there is no timeout and this method will block until the lock could be acquired
+        :param timeout: maximum wait time for acquiring the lock, ``None`` means use the default :attr:`~timeout`
+            is and if ``timeout < 0``, there is no timeout and this method will block until the lock could be acquired
         :param poll_interval: interval of trying to acquire the lock file
         :param poll_intervall: deprecated, kept for backwards compatibility, use ``poll_interval`` instead
-        :param blocking: defaults to True. If False, function will return immediately if it cannot obtain a lock on the
-         first attempt. Otherwise, this method will block until the timeout expires or the lock is acquired.
+        :param blocking: defaults to True. If False, function will return immediately if it cannot obtain a lock on
+            the first attempt. Otherwise, this method will block until the timeout expires or the lock is acquired.
         :raises Timeout: if fails to acquire lock within the timeout period
         :return: a context object that will unlock the file when the context is exited
 
@@ -355,7 +369,7 @@ class BaseFileLock(contextlib.ContextDecorator, metaclass=FileLockMeta):
         Releases the file lock. Please note, that the lock is only completely released, if the lock counter is 0.
         Also note, that the lock file itself is not automatically deleted.
 
-        :param force: If true, the lock counter is ignored and the lock is released in every case/
+        :param force: If true, the lock counter is ignored and the lock is released in every case.
 
         """
         if self.is_locked:
